
/* Chrysalide - Outil d'analyse de fichiers binaires
 * binary.c - équivalent Python du fichier "analysis/binary.h"
 *
 * Copyright (C) 2012-2017 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "binary.h"


#include <pygobject.h>


#include <i18n.h>


#include <analysis/binary.h>


#include "loaded.h"
#include "../access.h"
#include "../helpers.h"
#include "../format/executable.h"



/* Crée un nouvel objet Python de type 'LoadedBinary'. */
static PyObject *py_loaded_binary_new(PyTypeObject *, PyObject *, PyObject *);

/* Fournit le nom associé à l'élément binaire. */
static PyObject *py_loaded_binary_get_name(PyObject *, void *);

/* Fournit le format de fichier reconnu dans le contenu binaire. */
static PyObject *py_loaded_binary_get_format(PyObject *, void *);

/* Fournit le processeur de l'architecture liée au binaire. */
static PyObject *py_loaded_binary_get_processor(PyObject *, void *);

/* Fournit le tampon associé au contenu assembleur d'un binaire. */
static PyObject *py_loaded_binary_get_disassembled_cache(PyObject *, void *);



/******************************************************************************
*                                                                             *
*  Paramètres  : type = type de l'objet à instancier.                         *
*                args = arguments fournis à l'appel.                          *
*                kwds = arguments de type key=val fournis.                    *
*                                                                             *
*  Description : Crée un nouvel objet Python de type 'LoadedBinary'.          *
*                                                                             *
*  Retour      : Instance Python mise en place.                               *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static PyObject *py_loaded_binary_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    PyObject *result;                       /* Instance à retourner        */
    PyObject *format_obj;                   /* Objet pour le contenu       */
    int ret;                                /* Bilan de lecture des args.  */
    GExeFormat *format;                     /* Instance GLib correspondante*/
    GLoadedContent *binary;                 /* Version GLib du binaire     */

    ret = PyArg_ParseTuple(args, "O!", get_python_executable_format_type(), &format_obj);
    if (!ret) return NULL;

    format = G_EXE_FORMAT(pygobject_get(format_obj));

    g_object_ref(G_OBJECT(format));
    binary = g_loaded_binary_new(format);

    result = pygobject_new(G_OBJECT(binary));

    g_object_unref(binary);

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : self    = objet Python concerné par l'appel.                 *
*                closure = non utilisé ici.                                   *
*                                                                             *
*  Description : Fournit le nom associé à l'élément binaire.                  *
*                                                                             *
*  Retour      : Nom de fichier avec chemin absolu.                           *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static PyObject *py_loaded_binary_get_name(PyObject *self, void *closure)
{
    PyObject *result;                       /* Trouvailles à retourner     */
    GLoadedBinary *binary;                  /* Version native              */
    const char *name;                       /* Désignation du binaire      */

    binary = G_LOADED_BINARY(pygobject_get(self));

    name = g_loaded_binary_get_name(binary, true);

    result = PyUnicode_FromString(name);

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : self    = objet Python concerné par l'appel.                 *
*                closure = non utilisé ici.                                   *
*                                                                             *
*  Description : Fournit le format de fichier reconnu dans le contenu binaire.*
*                                                                             *
*  Retour      : Instance du format reconnu.                                  *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static PyObject *py_loaded_binary_get_format(PyObject *self, void *closure)
{
    PyObject *result;                       /* Instance Python à retourner */
    GLoadedBinary *binary;                  /* Binaire en cours d'analyse  */
    GExeFormat *format;                     /* Format du binaire lié       */

    binary = G_LOADED_BINARY(pygobject_get(self));
    format = g_loaded_binary_get_format(binary);

    result = pygobject_new(G_OBJECT(format));

    g_object_unref(G_OBJECT(format));

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : self    = objet Python concerné par l'appel.                 *
*                closure = non utilisé ici.                                   *
*                                                                             *
*  Description : Fournit le processeur de l'architecture liée au binaire.     *
*                                                                             *
*  Retour      : Instance du processeur associé.                              *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static PyObject *py_loaded_binary_get_processor(PyObject *self, void *closure)
{
    PyObject *result;                       /* Instance Python à retourner */
    GLoadedBinary *binary;                  /* Binaire en cours d'analyse  */
    GArchProcessor *proc;                   /* Architecture visée          */

    binary = G_LOADED_BINARY(pygobject_get(self));
    proc = g_loaded_binary_get_processor(binary);

    if (proc != NULL)
    {
        result = pygobject_new(G_OBJECT(proc));
        g_object_unref(G_OBJECT(proc));
    }
    else
    {
        result = Py_None;
        Py_INCREF(result);
    }

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : self    = classe représentant une instruction.               *
*                closure = adresse non utilisée ici.                          *
*                                                                             *
*  Description : Fournit le tampon associé au contenu assembleur d'un binaire.*
*                                                                             *
*  Retour      : Valeur associée à la propriété consultée.                    *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static PyObject *py_loaded_binary_get_disassembled_cache(PyObject *self, void *closure)
{
    PyObject *result;                       /* Trouvailles à retourner     */
    GLoadedBinary *binary;                  /* Version native              */
    GBufferCache *cache;                    /* Tampon à récupérer          */

    binary = G_LOADED_BINARY(pygobject_get(self));
    cache = g_loaded_binary_get_disassembled_cache(binary);

    if (cache != NULL)
    {
        result = pygobject_new(G_OBJECT(cache));
        g_object_unref(G_OBJECT(cache));
    }
    else
    {
        result = Py_None;
        Py_INCREF(result);
    }

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : -                                                            *
*                                                                             *
*  Description : Fournit un accès à une définition de type à diffuser.        *
*                                                                             *
*  Retour      : Définition d'objet pour Python.                              *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

PyTypeObject *get_python_loaded_binary_type(void)
{
    static PyMethodDef py_loaded_binary_methods[] = {
        { NULL }
    };

    static PyGetSetDef py_loaded_binary_getseters[] = {
        {
            "name", py_loaded_binary_get_name, NULL,
            "Name of the loaded binary.", NULL
        },
        {
            "format", py_loaded_binary_get_format, NULL,
            "File format recognized in the binary content.", NULL
        },
        {
            "processor", py_loaded_binary_get_processor, NULL,
            "Handler for the current binary processor.", NULL
        },
        {
            "disassembled_cache", py_loaded_binary_get_disassembled_cache, NULL,
            "Disassembled buffer cache.", NULL
        },
        { NULL }
    };

    static PyTypeObject py_loaded_binary_type = {

        PyVarObject_HEAD_INIT(NULL, 0)

        .tp_name        = "pychrysalide.analysis.LoadedBinary",

        .tp_flags       = Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,

        .tp_doc         = "PyChrysalide loaded binary",

        .tp_methods     = py_loaded_binary_methods,
        .tp_getset      = py_loaded_binary_getseters,
        .tp_new         = py_loaded_binary_new

    };

    return &py_loaded_binary_type;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : -                                                            *
*                                                                             *
*  Description : Prend en charge l'objet 'pychrysalide.analysis.LoadedBinary'.*
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool ensure_python_loaded_binary_is_registered(void)
{
    PyTypeObject *type;                     /* Type Python 'ProxyFeeder'   */
    PyObject *module;                       /* Module à recompléter        */
    PyObject *dict;                         /* Dictionnaire du module      */

    type = get_python_loaded_binary_type();

    if (!PyType_HasFeature(type, Py_TPFLAGS_READY))
    {
        module = get_access_to_python_module("pychrysalide.analysis");

        dict = PyModule_GetDict(module);

        if (!ensure_python_loaded_content_is_registered())
            return false;

        if (!register_class_for_pygobject(dict, G_TYPE_LOADED_BINARY, type, &PyGObject_Type))
            return false;

    }

    return true;

}
